from .filters import Filter

class Query:
    """Dork query builder"""
    def __init__(self) -> None:
        self._query = ""
        self._query_size = 0

    def _append_query_string(self, subquery: str) -> None:
        """Append given subquery to self._query (used internal, don't call)

        Args:
            subquery (str)
        """
        sub = ""
        if self._query_size > 0:
            sub += " "
        sub += subquery
        self._query += sub
        self._query_size += 1

    def _build_subquery_string(self, filter: str, keywords: list, group_seperator: str, hide: bool = False) -> str:
        sub = ""
        if hide:
            sub += "-"
        if type(keywords) == str:
            sub += f"{filter}{keywords}"
        elif type(keywords) == list:
            sub += filter
            sub += self._build_group_string(keywords, group_seperator)
        return sub

    def _build_group_string(self, keywords: list, group_seperator: str) -> str:
        return f"({f' {group_seperator} '.join(keywords)})"

    def allintext(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for occurrences of all the keywords given."""
        sub = self._build_subquery_string(Filter.ALLINTEXT, keywords, group_seperator, hide)
        self._append_query_string(sub)
    
    def intext(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for the occurrences of keywords all at once or one at a time."""
        sub = self._build_subquery_string(Filter.INTEXT, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def inurl(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for a URL matching one of the keywords."""
        sub = self._build_subquery_string(Filter.INURL, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def allinurl(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for a URL matching all the keywords."""
        sub = self._build_subquery_string(Filter.ALLINURL, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def intitle(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for occurrences of keywords in title all or one."""
        sub = self._build_subquery_string(Filter.INTITLE, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def allintitle(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for occurrences of keywords in the title all at a time."""
        sub = self._build_subquery_string(Filter.ALLINTITLE, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def site(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Specifically searches that particular site and lists all the results for that site."""
        sub = self._build_subquery_string(Filter.SITE, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def filetype(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for a particular filetype mentioned in the query."""
        sub = self._build_subquery_string(Filter.FILETYPE, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def link(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for external links to pages."""
        sub = self._build_subquery_string(Filter.LINK, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def numrange(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Used to locate specific numbers in your searches."""
        sub = self._build_subquery_string(Filter.NUMRANGE, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def before(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Used to search before a particular date."""
        sub = self._build_subquery_string(Filter.BEFORE, keywords, group_seperator, hide)
        self._append_query_string(sub)
    
    def after(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Used to search before a particular date."""
        sub = self._build_subquery_string(Filter.AFTER, keywords, group_seperator, hide)
        self._append_query_string(sub)
    
    def allinanchor(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """This shows sites which have the keyterms in links pointing to them, in order of the most links."""
        sub = self._build_subquery_string(Filter.ALLINANCHOR, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def inanchor(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """This shows sites which have the keyterms in links pointing to them, in order of the most links."""
        sub = self._build_subquery_string(Filter.INANCHOR, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def allinpostauthor(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Exclusive to blog search, this one picks out blog posts that are written by specific individuals."""
        sub = self._build_subquery_string(Filter.ALLINPOSTAUTHOR, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def inpostauthor(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Exclusive to blog search, this one picks out blog posts that are written by specific individuals."""
        sub = self._build_subquery_string(Filter.INPOSTAUTHOR, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def related(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """List web pages that are “similar” to a specified web page."""
        sub = self._build_subquery_string(Filter.RELATED, keywords, group_seperator, hide)
        self._append_query_string(sub)
    
    def cache(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Shows the version of the web page that Google has in its cache."""
        sub = self._build_subquery_string(Filter.CACHE, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def ext(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Searches for a particular filetype mentioned in the query."""
        sub = self._build_subquery_string(Filter.EXT, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def symbol_and(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Adds a & symbol."""
        sub = self._build_subquery_string(Filter.AND, keywords, group_seperator, hide)
        self._append_query_string(sub)
    
    def written_and(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Adds an AND."""
        sub = self._build_subquery_string(Filter.AND2, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def symbol_or(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Adds a | symbol."""
        sub = self._build_subquery_string(Filter.OR, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def written_or(self, keywords: list, hide: bool = False, group_seperator: str = "|") -> None:
        """Adds an OR."""
        sub = self._build_subquery_string(Filter.OR2, keywords, group_seperator, hide)
        self._append_query_string(sub)

    def keyword_group(self, keywords: list, group_seperator: str = "|", hide: bool = False) -> None:
        """Adds a list of keywords to the query."""
        sub = self._build_subquery_string("", keywords, group_seperator, hide)
        self._append_query_string(sub)

    def raw(self, raw: str) -> None:
        """Add a raw string to the query"""
        self._append_query_string(raw)

    @property
    def query(self) -> str:
        return self._query

    def __str__(self):
        return self.query


class Filter:
    """Dork filters"""
    ALLINTEXT = "allintext:" # Searches for occurrences of all the keywords given.
    INTEXT = "intext:" # Searches for the occurrences of keywords all at once or one at a time.
    INURL = "inurl:" # Searches for a URL matching one of the keywords.
    ALLINURL = "allinurl:" # Searches for a URL matching all the keywords in the query.
    INTITLE = "intitle:" # Searches for occurrences of keywords in title all or one.
    ALLINTITLE = "allintitle:" # Searches for occurrences of keywords all at a time.
    SITE = "site:" # Specifically searches that particular site and lists all the results for that site.
    FILETYPE = "filetype:" # Searches for a particular filetype mentioned in the query.
    LINK = "link:" # Searches for external links to pages.
    NUMRANGE = "numrange:" # Used to locate specific numbers in your searches.
    BEFORE = "before:" # Used to search before a particular date.
    AFTER = "after:" # Used to search before a particular date.
    ALLINANCHOR = "allinanchor:" # This shows sites which have the keyterms in links pointing to them, in order of the most links.
    INANCHOR = "inanchor:" # This shows sites which have the keyterms in links pointing to them, in order of the most links.
    ALLINPOSTAUTHOR = "allinpostauthor:" # Exclusive to blog search, this one picks out blog posts that are written by specific individuals.
    INPOSTAUTHOR = "inpostauthor:" # Exclusive to blog search, this one picks out blog posts that are written by specific individuals.
    RELATED = "related:" # List web pages that are “similar” to a specified web page.
    CACHE = "cache:" # Shows the version of the web page that Google has in its cache.
    EXT = "ext:" # Searches for a particular filetype mentioned in the query.
    AND = "&"
    OR = "|"
    AND2 = "AND"
    OR2 = "OR"


from .searcher import Searcher
from .query import Query
from .sites import Site
from .extension import Extension

def search(query: Query) -> list:
    """Wrapper around searcher.search"""
    s = Searcher()
    return s.search(query)

def index_of(folder: str) -> list:
    """Search exposed folders"""
    q = Query()
    q.allintitle(f"index of /{folder}")
    results = []
    for r in search(q):
        results.append(r)
    return results

def leak(query: str) -> list:
    """Search leaks"""
    results = []
    for s in Site.LIST_LEAK:
        q = Query()
        q.raw(query)
        q.site(s)
        print(q)
        r = search(q)
        results.extend(r)
    for s in Site.LIST_PASTING:
        q = Query()
        q.raw(query)
        q.site(s)
        print(q)
        r = search(q)
        results.extend(r)
    for s in Site.LIST_FILESHARING:
        q = Query()
        q.raw(query)
        q.site(s)
        print(q)
        r = search(q)
        results.extend(r)
    return results

def presentation(query: str) -> list:
    results = []
    q = Query()
    q.intitle(query)
    q.filetype([Extension.PPTX, Extension.PDF])
    r = search(q)
    results.extend(r)
    return results

def document(query: str) -> list:
    results = []
    q = Query()
    q.intitle(query)
    q.filetype([Extension.PDF, Extension.DOCX])
    r = search(q)
    results.extend(r)
    return results


# cli.py

import click

from .wrapper import search as wrapper_search
from .wrapper import leak as wrapper_leak
from .wrapper import index_of as wrapper_index
from .wrapper import presentation as wrapper_presentation
from .wrapper import document as wrapper_document

LINKVERTISE_MESSAGE = "---\n!!! Redirected to linkvertise.com? Use 'thebypasser.com' to bypass linkvertise links! !!!\n---"

@click.command()
@click.option("-l", "--leak", default=False, required=False, is_flag=True)
@click.option("-p", "--presentation", default=False, required=False, is_flag=True)
@click.option("-i", "--index", default=False, required=False, is_flag=True)
@click.option("-d", "--document", default=False, required=False, is_flag=True)
@click.argument("query")
def cli(leak, presentation, document, env, index, query) -> None:
    if (leak):
        click.echo(LINKVERTISE_MESSAGE)
        results = wrapper_leak(query)
    elif (index):
        results = wrapper_index(query)
    elif (document):
        results = wrapper_document(query)
    elif (presentation):
        results = wrapper_presentation(query)
    else:
        results = wrapper_search(query)
    
    if (len(results) > 0):
        for url in results:
            click.echo(f"[+] {url}")
    else:
        click.echo("No results. Try a different method/query or use a VPN.")
if __name__ == "__main__":
    cli()


import requests
from bs4 import BeautifulSoup

from .engine import Engine
from .query import Query

class Searcher:
    def __init__(self, engine: str = Engine.GOOGLECOM):
        """
        Args:
            engine (str): query url from a searchengine (ex. Engine.GOOGLECOM)
        """
        self._engine = engine

    def _request(self, query: str) -> str:
        headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36'
        }
        res = requests.get(self._engine,
        headers=headers,
        params=dict(q=query)
        )
        res.raise_for_status()
        return res.text

    def _parse(self, html: str) -> list:
        soup = BeautifulSoup(html, "html.parser")
        blocks = soup.find_all("div", attrs={"class": "g"})
        links = []
        for result in blocks:
            element = result.find("a", href=True)
            links.append(element["href"])
        return links

    def search(self, query: Query) -> list:
        """Start a dork search with given query

        Args:
            
            query (str): dork query

        Returns:
            list: list of results
        """
        if type(query) == Query:
            query_string = query.query
        else:
            query_string = query
        
        html = self._request(query_string)
        links = self._parse(html)
        return links


class Engine:
    """Searchengine URL's"""
    GOOGLECOM = "https://google.com/search"
    GOOGLEDE = "https://google.de/search"
    GOOGLEFR = "https://google.fr/search"
    GOOGLEES = "https://google.es/search"


class Extension:
    # predefined filetypes for fast use
    PDF = "pdf"
    # word
    DOC = "doc"
    DOT = "dot"
    WBK = "wbk"
    DOCX = "docx"
    DOTX = "dotx"
    DOTM = "dotm"
    DOCB = "docb"
    WLL = "wll"
    WWL = "wwl"
    # excel
    XLS = "xls"
    XLT = "xlt"
    XLM = "xlm"
    XLL_ = "xll_"
    XLA_ = "xla_"
    XLA5 = "xla5"
    XLA8 = "xla8"
    XLSX = "xlsx"
    XLSM = "xlsm"
    XLTS = "xltx"
    XLTM = "xltm"
    XLSB = "xlsb"
    XLA = "xla"
    XLAM = "xlam"
    XLL = "xll"
    XLW = "xlw"
    # powepoint
    PPT = "ppt"
    POT = "pot"
    PPS = "pps"
    PPA = "ppa"
    PPAM = "ppam"
    PPTX = "pptx"
    PPTM = "pptm"
    POTX = "potx"
    POTM = "potm"
    PPAM = "ppam"
    PPSX = "ppsx"
    PPSM = "ppsm"
    SLDX = "sldx"
    SLDM = "sldm"
    PA = "pa"
    # videos
    MOV = "mov"
    MP4 = "mp4"
    WMV = "wmv"
    AVI = "avi"
    AVCHD = "avchd"
    FLV = "flv"
    F4V = "f4v"
    SWF = "swf"
    MKV = "mkv"
    WEBM = "webm"
    OGG = "ogg"
    OGV = "ogv"
    MPG = "mpg"
    AAC = "aac"
    WMA = "wma"
    # images
    JPG = "jpg"
    JPEG = "jpeg"
    JIFF = "jiff"
    PJPEG = "pjpeg"
    PJP = "pjp"
    PNG = "png"
    SVG = "svg"
    WEBP = "webp"
    GIF = "gif"
    BMP = "bmp"
    ICO = "ico"
    CUR = "cur"
    TIF = "tif"
    TIFF = "tiff"
    AVIF = "avif"
    APNG = "apng"

    ENV = "env"
    # lists
    LIST_IMAGE = [JPG, JPEG, JIFF, PJPEG, PJP, PNG, SVG, WEBP, GIF, BMP, ICO, CUR, TIF, TIFF, AVIF, APNG]
    LIST_VIDEO = [MOV, MP4, WMV, AVI, AVCHD, FLV, F4V, SWF, MKV, WEBM, OGG, OGV, MPG, AAC, WMA]
    LIST_POWERPOINT = [PPT, POT, PPS, PPA, PPAM, PPTX, PPTM, POTX, POTM, PPAM, PPSX, PPSM, SLDX, SLDM, PA]
    LIST_EXCEL = [XLS, XLT, XLM, XLL_, XLA_, XLA5, XLA8, XLSX, XLSM, XLTS, XLSB, XLA, XLAM, XLL, XLW]
    LIST_WORD = [DOC, DOT, WBK, DOCX, DOTX, DOTM, DOCB, WLL, WWL]


import os
import sys
from setuptools import setup, find_packages

# --- version check ---
CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 6)
if CURRENT_PYTHON < REQUIRED_PYTHON:
    print(
f"""
--- Unsopported Python version ---
Ventus requires at least Python {REQUIRED_PYTHON[0]}.{REQUIRED_PYTHON[1]},
but you're trying to install it on Python {CURRENT_PYTHON[0]}.{CURRENT_PYTHON[1]}.
"""
    )
    sys.exit(1)

# --- publish option ---
if sys.argv[-1] == 'publish':
    """pack and publish"""
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()

# --- read readme.md ---
with open("README.md", "r") as f:
    readme = f.read()

# --- setting up ---
setup(
    name="ventus",
    version='0.3.1',
    author="aaronlyy (Aaron Levi)",
    author_email="<aaronlevican@gmail.com>",
    url="https://github.com/aaronlyy/ventus",
    description='A google dorking library and cli.',
    long_description_content_type="text/markdown",
    long_description=readme,
    packages=["ventus"],
    entry_points={
        "console_scripts": [
            "ventus = ventus.cli:cli"
        ]
    },
    install_requires=['requests', 'beautifulsoup4', 'click'],
    keywords=['dorking', 'google', 'scraping', 'google dorking', 'hacking', 'cracking'],
    classifiers=[
        "Intended Audience :: Developers",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development :: Libraries',
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)